#include "Defines.h"
#include "Types.h"
#include <ioavr.h>
#include <intrinsics.h>

extern Response      IO_Buffer[BUFER_LENGTH+1];
extern unsigned char Marker[BUFER_LENGTH+1];
extern unsigned int Get_CRC16(unsigned char *puchMsg, unsigned char usDataLen);
extern void Response_through_USB(void *p);


void ADC_init(void) {
  
#if defined (__AT90USB1287__)
  
// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: AREF pin
// ADC High Speed Mode: Off
// Digital input buffers on ADC0: On, ADC1: On, ADC2: On, ADC3: On
// ADC4: On, ADC5: On, ADC6: On, ADC7: On
DIDR0=0x00;
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x86;
ADCSRB&=0x7F;

#else

// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: AREF pin
// ADC High Speed Mode: Off
// Digital input buffers on ADC0: On, ADC1: On, ADC2: On, ADC3: On
// ADC4: On, ADC5: On, ADC6: On, ADC7: On
DIDR0=0x00;
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x87;
ADCSRB&=0x7F;

#endif
}

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
  unsigned int adc_buf[ADC_BUFER_LENGTH];
  unsigned int ADC_result;
  unsigned char i, num_max, num_min;
  
	ADMUX = adc_input | (ADC_VREF_TYPE & 0xff);

// Delay needed for the stabilization of the ADC input voltage
#if defined (__AT90USB1287__)
	__delay_cycles(80); // 10 us on 8 MHz
#else
	__delay_cycles(160); // 10 us on 16 MHz
#endif

for(i=0;i<ADC_BUFER_LENGTH;i++) {
		// Start the AD conversion
	ADCSRA|=0x40;
		// Wait for the AD conversion to complete
	__watchdog_reset(); 
	while ((ADCSRA & 0x10)==0);
	ADCSRA|=0x10;
	adc_buf[i] = ADC; // Save result
	}

ADC_result = adc_buf[0]; 
num_max = 0;
for(i=0;i<ADC_BUFER_LENGTH;i++) { // Search of the maximal element of the buffer.
  if(adc_buf[i] > ADC_result) { 
		ADC_result = adc_buf[i];
		num_max = i;
    	}
}

ADC_result = adc_buf[0]; 
num_min = 0;
for(i=0;i<ADC_BUFER_LENGTH;i++) { // Search of the minimal element of the buffer.
  if(adc_buf[i] < ADC_result) { 
		ADC_result = adc_buf[i];
		num_min = i;
    	}
}

ADC_result = 0;
adc_buf[num_max] = 0;
adc_buf[num_min] = 0;

for(i=0;i<ADC_BUFER_LENGTH;i++) { ADC_result += adc_buf[i]; } // Calculation of the sum.

	if(num_max == num_min) {
  		ADC_result = (unsigned int)(ADC_result / (ADC_BUFER_LENGTH-1)); // Reception of average value.
	} else {
		ADC_result = (unsigned int)(ADC_result / (ADC_BUFER_LENGTH-2)); // Reception of average value.
	}
return ADC_result;
}


unsigned char Read_ADC_Channels(unsigned char adc_lay)
{
unsigned char i, j;
unsigned int result;
	
	IO_Buffer[adc_lay].Length = 16;

	for (j = i = 0; i < 8; i++) {
		result = read_adc(i);
		IO_Buffer[adc_lay].Data[j] = ((result >> 8) & 0xff);
		IO_Buffer[adc_lay].Data[j+1] = ((result) & 0xff); 
		j += 2;
	}
 
	return 1;
}
