/*
27.02.08 ST
This file contains functions of input and output, 
calculation CRC, etc. tasks of test board Apollo.
*/

#include "Defines.h"
#include "Types.h"

#include "conf\config.h"
#include "conf\conf_usb.h"
#include "cdc_task.h"
#include "..\..\lib_mcu\usb\usb_drv.h"
#include "usb_descriptors.h"
#include "..\..\modules\usb\device_chap9\usb_standard_request.h"
#include "usb_specific_request.h"
#include "..\..\..\common\lib_mcu\uart\uart_lib.h"
#include "TWI_slave.h"
#include <string.h>

#include "uart_usb_lib.h"

#include "led.h"

// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20




unsigned int Get_CRC16(unsigned char *puchMsg, unsigned char usDataLen);
unsigned char Get_Free_Layer(unsigned char offset);
extern int uart_usb_putchar(int data_to_send);
extern void uart_usb_flush (void);
extern void DelayMs(unsigned short Ms);
void cdc_task(void);
uchar USB_manager(void);
void markers_manager(void);
VERSION_MAS;
extern unsigned char my_address;
//extern unsigned char opponent_address;
extern unsigned char i2c_error;     

unsigned char counter_for_I2C=0;
unsigned char layer_action_for_I2C=0;
unsigned char temp_for_I2C=0;

extern void i2c_init(unsigned char my_addr);
extern void i2c_start(void);
extern void i2c_stop(void);
extern void i2c_tx(unsigned char byte);
extern void i2c_tx_addr(unsigned char adr, unsigned char rw); // TEST!
extern unsigned char i2c_rx(unsigned char last_byte);

//extern void External_RAM_Init(void);
extern unsigned char Read_From_Ext_Memory(unsigned char lay);
extern unsigned char Write_To_Ext_Memory(unsigned char lay);

extern unsigned char Match_Test(unsigned char lay);
extern unsigned char Ext_Flash_Read(unsigned char lay);
extern unsigned char Get_Flash_ID(unsigned char lay);

extern unsigned int read_adc(unsigned char adc_input);
extern unsigned char Read_ADC_Channels(unsigned char adc_lay);
extern unsigned char Set_ATPWR_pins(unsigned char lay);
extern unsigned char Set_FAN_Frequency(unsigned char lay);

unsigned char Ext_I2C_Read(unsigned char lay);
unsigned char Ext_I2C_Write(unsigned char lay);
unsigned char Ext_I2C_Result(unsigned char lay);

extern unsigned char Ext_PIRO_Read(unsigned char lay);

unsigned char DebugCmd(unsigned char lay);

//Response      Apollo_Response;
unsigned char Marker[BUFER_LENGTH+1];
Response      IO_Buffer[BUFER_LENGTH+1];

unsigned char layer_action = 0;
unsigned char temp_buffer = 0;
unsigned char temp_cdc=0;

void Migat_Raz(char p) {
 uchar temp;
 	for (temp=0; temp<(p); temp++) {
		Leds_on();
		DelayMs(250);
		Leds_off();
		DelayMs(250);
	} 
}


void Response_through_USB(void *p)
{
uchar i,temp;
uint crc_buf = 0;
Response *pointer = (Response*)p;
 
	temp = pointer->Length;
	uart_usb_putchar(pointer->Address);
	
	if( (pointer->Sender) == 0 ) { pointer->Sender = my_address; }
	
	uart_usb_putchar(pointer->Sender);
	uart_usb_putchar(pointer->Length);
	uart_usb_putchar(pointer->Cmd);
	
	for (i=0; i<temp; i++)
		uart_usb_putchar(pointer->Data[i]);
	
	crc_buf = Get_CRC16(&(pointer->Address), pointer->Length);

	uart_usb_putchar((crc_buf >> 8)&0xff);
	uart_usb_putchar(crc_buf & 0xff);
	
	uart_usb_flush();
}

void Response_through_I2C(void *p, unsigned char Addr)
{
uchar i,temp;
uint i2c_crc_buf = 0;
Response *pointer = (Response*)p;

	i2c_start();
     
	  i2c_tx_addr((0x7F & Addr), 'W');
 
	temp = pointer->Length;

	i2c_tx(pointer->Address);
	
	pointer->Sender = my_address;	  
	
	i2c_tx(pointer->Sender);
	i2c_tx(pointer->Length);
	i2c_tx(pointer->Cmd);
	
	for (i=0; i<temp; i++) 	i2c_tx(pointer->Data[i]);
	
		i2c_crc_buf = Get_CRC16(&(pointer->Address), pointer->Length);

	i2c_tx((i2c_crc_buf >> 8)&0xff);
	i2c_tx(i2c_crc_buf & 0xff);
	
	i2c_stop();
}

void Router_of_messages(uchar layer)
{
	Leds_off();
	Marker[layer] = FREE;
}


unsigned char Emergency_message(unsigned char message)
{
uint temp;
uchar em_lay;
em_lay = Get_Free_Layer(EXPANDED);

	 
		IO_Buffer[em_lay].Address = (my_address & 0x80); 
        IO_Buffer[em_lay].Sender = 0; 
		IO_Buffer[em_lay].Length = 2; 
		IO_Buffer[em_lay].Cmd = ERROR_MESSAGE;
		IO_Buffer[em_lay].Data[0] = message;
		IO_Buffer[em_lay].Data[1] = 0;
			temp = Get_CRC16(&IO_Buffer[em_lay].Address, IO_Buffer[em_lay].Length);
		IO_Buffer[em_lay].CRC_H = (temp >> 8)&0xFF;
		IO_Buffer[em_lay].CRC_L = temp & 0xFF;
	
	return 0xFF; 
}

unsigned char Version_message(unsigned char lay)
{
	    IO_Buffer[lay].Length = 4;
		
		IO_Buffer[lay].Data[0] = (version[12] - 48); 
		IO_Buffer[lay].Data[1] = (version[13] - 48);
	        IO_Buffer[lay].Data[2] = (version[14] - 48);
		IO_Buffer[lay].Data[3] = (version[15] - 48);
		
	return 0xFF; 
}


/* Table of CRC values for highorder byte */
__flash unsigned char auchCRCHi[256] = {
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01,
0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41,
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81,
0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01,
0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01,
0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01,
0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40
};

/* Table of CRC values for loworder byte */
__flash char auchCRCLo[256] = {
0x00, 0xC0, 0xC1, 0x01, 0xC3, 0x03, 0x02, 0xC2, 0xC6, 0x06, 0x07, 0xC7, 0x05, 0xC5, 0xC4,
0x04, 0xCC, 0x0C, 0x0D, 0xCD, 0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09,
0x08, 0xC8, 0xD8, 0x18, 0x19, 0xD9, 0x1B, 0xDB, 0xDA, 0x1A, 0x1E, 0xDE, 0xDF, 0x1F, 0xDD,
0x1D, 0x1C, 0xDC, 0x14, 0xD4, 0xD5, 0x15, 0xD7, 0x17, 0x16, 0xD6, 0xD2, 0x12, 0x13, 0xD3,
0x11, 0xD1, 0xD0, 0x10, 0xF0, 0x30, 0x31, 0xF1, 0x33, 0xF3, 0xF2, 0x32, 0x36, 0xF6, 0xF7,
0x37, 0xF5, 0x35, 0x34, 0xF4, 0x3C, 0xFC, 0xFD, 0x3D, 0xFF, 0x3F, 0x3E, 0xFE, 0xFA, 0x3A,
0x3B, 0xFB, 0x39, 0xF9, 0xF8, 0x38, 0x28, 0xE8, 0xE9, 0x29, 0xEB, 0x2B, 0x2A, 0xEA, 0xEE,
0x2E, 0x2F, 0xEF, 0x2D, 0xED, 0xEC, 0x2C, 0xE4, 0x24, 0x25, 0xE5, 0x27, 0xE7, 0xE6, 0x26,
0x22, 0xE2, 0xE3, 0x23, 0xE1, 0x21, 0x20, 0xE0, 0xA0, 0x60, 0x61, 0xA1, 0x63, 0xA3, 0xA2,
0x62, 0x66, 0xA6, 0xA7, 0x67, 0xA5, 0x65, 0x64, 0xA4, 0x6C, 0xAC, 0xAD, 0x6D, 0xAF, 0x6F,
0x6E, 0xAE, 0xAA, 0x6A, 0x6B, 0xAB, 0x69, 0xA9, 0xA8, 0x68, 0x78, 0xB8, 0xB9, 0x79, 0xBB,
0x7B, 0x7A, 0xBA, 0xBE, 0x7E, 0x7F, 0xBF, 0x7D, 0xBD, 0xBC, 0x7C, 0xB4, 0x74, 0x75, 0xB5,
0x77, 0xB7, 0xB6, 0x76, 0x72, 0xB2, 0xB3, 0x73, 0xB1, 0x71, 0x70, 0xB0, 0x50, 0x90, 0x91,
0x51, 0x93, 0x53, 0x52, 0x92, 0x96, 0x56, 0x57, 0x97, 0x55, 0x95, 0x94, 0x54, 0x9C, 0x5C,
0x5D, 0x9D, 0x5F, 0x9F, 0x9E, 0x5E, 0x5A, 0x9A, 0x9B, 0x5B, 0x99, 0x59, 0x58, 0x98, 0x88,
0x48, 0x49, 0x89, 0x4B, 0x8B, 0x8A, 0x4A, 0x4E, 0x8E, 0x8F, 0x4F, 0x8D, 0x4D, 0x4C, 0x8C,
0x44, 0x84, 0x85, 0x45, 0x87, 0x47, 0x46, 0x86, 0x82, 0x42, 0x43, 0x83, 0x41, 0x81, 0x80,
0x40
};

unsigned int Get_CRC16(unsigned char *puchMsg, unsigned char usDataLen)
{
  unsigned char uchCRCHi = 0xFF;  /* high byte of CRC initialized */
  unsigned char uchCRCLo = 0xFF;  /* low byte of CRC initialized */
  unsigned char uIndex;           /* will index into CRC lookup table */
  usDataLen += (HEADER_LENGTH_MAX - 2);

    while ( usDataLen-- )   /* pass through message buffer */
    {
     uIndex = uchCRCHi ^ *puchMsg++;  /* calculate the CRC */
     uchCRCHi = uchCRCLo ^ auchCRCHi[uIndex];
     uchCRCLo = auchCRCLo[uIndex];
    }
return ((uchCRCHi << 8) | uchCRCLo);
}

unsigned char Set_Address(uchar x, uchar y, uchar chan)
{  
  
  if ( (x < 4) && (y < 31) ) {
                              return ((x + (y << 2)) | (chan << 1)); 
                             } 
  else { return Emergency_message(INVALID_ADDRESS); }
}


unsigned char Get_Free_Layer(unsigned char offset)
{
unsigned char ii;
	
	__disable_interrupt();
	for(ii=0;ii<BUFER_LENGTH+offset;ii++) {
		if(Marker[ii] == FREE) {
			Marker[ii] = OCCUPIED;
			__enable_interrupt();
			return ii;
		}
	}
	__enable_interrupt();
	return BUFFER_FULL;
}

unsigned char Execute_a_command(unsigned char exe_lay)
{
unsigned char reti = 0;
  
	switch (IO_Buffer[exe_lay].Cmd) {
		case VERSION_MESSAGE:
			 reti = Version_message(exe_lay);
			 break;
                /*         
		case WRITE_REGISTERS:
			 reti = Write_To_Ext_Memory(exe_lay);
			 break;
		case READ_REGISTERS:
			 reti = Read_From_Ext_Memory(exe_lay); 
			 break;
		case ADC_READ: 
			 reti = Read_ADC_Channels(exe_lay);
			 break;
		case ATPWR_SET: 
			 reti = Set_ATPWR_pins(exe_lay);  
			 break;
		case FAN_SET:
			 reti = Set_FAN_Frequency(exe_lay);
			 break;
		case EXT_I2C_READ:
			 reti = Ext_I2C_Read(exe_lay);
			 break;
		case EXT_I2C_WRIGHT:
			 reti = Ext_I2C_Write(exe_lay);
			 break;
		case EXT_I2C_RESULT:
			 reti = Ext_I2C_Result(exe_lay);
			 break;
			 
		case DEBUG_CMD:
			 reti = DebugCmd(exe_lay);
			 break;
		*/	 
		case PIRO_CMD:
			 reti = Ext_PIRO_Read(exe_lay);
			 break;
                /*         
		case FLASH_REC_CMD:
			 reti = Match_Test(exe_lay);
			 break;
			 
		case FLASH_PLY_CMD:
			 reti = Ext_Flash_Read(exe_lay);
			 break;    
                         
		case FLASH_ID_CMD:
			 reti = Get_Flash_ID(exe_lay);
			 break;                             
                */         
		default:
			 Emergency_message(INVALID_CMD);
			 reti = 0;
			 break;
		};
	return reti;
}

uchar USB_manager(void)
{
unsigned char i, Len_mes, k;

	i = Get_Free_Layer(STANDARD);
	
	if (i == BUFFER_FULL) { 
		Emergency_message(BUFFER_FULL);
		return 0;
	}
	
	IO_Buffer[i].Address = uart_usb_getchar();
	IO_Buffer[i].Sender  = uart_usb_getchar();
	Len_mes = uart_usb_getchar();
	IO_Buffer[i].Length = Len_mes;
	IO_Buffer[i].Cmd = uart_usb_getchar();
	
	for (k=0;k<Len_mes;k++)
		IO_Buffer[i].Data[k] = uart_usb_getchar();
	
	IO_Buffer[i].CRC_H = uart_usb_getchar();
	IO_Buffer[i].CRC_L = uart_usb_getchar();
        
	Marker[i] = USB_chanel; // TEMP
        /*
	if( ((IO_Buffer[i].CRC_H << 8) | IO_Buffer[i].CRC_L) == 
	Get_CRC16(&IO_Buffer[i].Address, Len_mes) ) { 
	
	Marker[i] = USB_chanel;
	
	} else {
	Emergency_message(INVALID_CRC); 
	Marker[i] = FREE;
	}
        */
return 1;
}

void markers_manager(void) {
//	unsigned char temp;
//unsigned int temp_CRC;
  
	if (layer_action >= BUFER_LENGTH)
		layer_action = 0; 
	else {
		switch (Marker[layer_action]) {
			case FREE:
			case OCCUPIED:
				__no_operation();
				break;
									  
			case USB_chanel: // message from USB
			  
				if ((0x7F & IO_Buffer[layer_action].Address) == my_address) {
				  
				 		Execute_a_command(layer_action);
						Response_through_USB(&IO_Buffer[layer_action]);
                		Marker[layer_action] = FREE;
				  
						} else {
						
						//IO_Buffer[layer_action].Sender = my_address;  
						Response_through_I2C(&IO_Buffer[layer_action], IO_Buffer[layer_action].Address);
						Marker[layer_action] = FREE;
	    				} 
				break;
										
			case I2C_chanel: // message from I2C
			  
				//if ( (0x7F & IO_Buffer[layer_action].Address) == my_address ) {
			  if(1) { // TEST!!!!!!!!!!
					if( Is_device_enumerated() ) { // USB enumerated? Device is MASTER!
													Response_through_USB(&IO_Buffer[layer_action]);
													Marker[layer_action] = FREE;
													//  Migat_Raz(3); 
													
						} else { // USB NOT enumerated? Device is SLAVE!
					  			if ( (0x80 & IO_Buffer[layer_action].Address) ) { // if MSB bit set
											 	Execute_a_command(layer_action); 
											 	Response_through_I2C(&IO_Buffer[layer_action], IO_Buffer[layer_action].Sender);
												//Marker[layer_action] = FREE;
												}
						Marker[layer_action] = FREE;
						//Migat_Raz(5); 
                      //
					}
				} else { // not my address
					// Router_of_messages(layer_action);
				  Marker[layer_action] = FREE;
				}
				break;
			
			case SPI_chanel: // message from SPI
				__no_operation();
				break;
			
			default:
				Emergency_message(INVALID_TARGET);
				Marker[layer_action] = FREE;
		} // end of switch
			
		layer_action++; 
	} 
}

uchar TWI_manager(void) {
    char chAddr, chLen, chSender, i, k;
//	unsigned char temp, a=0;
//	unsigned char buf[10];
	
	
if(TWCR&(1<<TWINT)) {
   
   if((TWSR == TWI_SRX_GEN_ACK) || (TWSR == TWI_SRX_ADR_ACK)) {
   	chAddr = i2c_rx(4);
	
		  	i = Get_Free_Layer(STANDARD);

	chSender = i2c_rx(4);
	chLen = i2c_rx(4);
	
	IO_Buffer[i].Address = chAddr;
    IO_Buffer[i].Sender = chSender;
	IO_Buffer[i].Length = chLen;
	IO_Buffer[i].Cmd = i2c_rx(4);

	for (k=0;k<chLen;k++) IO_Buffer[i].Data[k] = i2c_rx(4);
	
	IO_Buffer[i].CRC_H = i2c_rx(4);
	IO_Buffer[i].CRC_L = i2c_rx(4);
	
	
	if( ((IO_Buffer[i].CRC_H << 8) | IO_Buffer[i].CRC_L) == 
	Get_CRC16(&IO_Buffer[i].Address, chLen) ) { 
	//if(1) {
	Marker[i] = I2C_chanel;
	
	
	///////////////////////test
	//	Migat_Raz(8);
	//////////////////test  

	} else {
	///////////////////////test 
	Migat_Raz(0x7F & chAddr);
	//////////////////test  
	//Emergency_message(INVALID_CRC); 
	Marker[i] = FREE;
	}
/////////////////////////////////////	
		i2c_stop();	
	   
   } 
}
	//i2c_stop();	
return 1;	
}	

unsigned char DebugCmd(unsigned char lay)
{
	IO_Buffer[lay].Length = 0;
	return 1; 
}
