#ifndef __DEFINE_H_
#define __DEFINE_H_

/* define.h  common constant definition header */
/*

*/

#if defined (__AT90USB1287__)
    #if (TARGET_PLAT == PIROPILOT)
	#define	F_CLK						16000000
	#define TWI_CLOCK_REGISTER_SETUP	0x30	// 100 kHz with 16 MHz quartz
    #else
        #define	F_CLK						8000000
        #define TWI_CLOCK_REGISTER_SETUP	0x1C	// 100 kHz with 8 MHz quartz
    #endif
#else
	#define	F_CLK						16000000
	#define TWI_CLOCK_REGISTER_SETUP	0x30	// 100 kHz with 16 MHz quartz
#endif

/* RAM defines  */
#define MEMORY_MAPPED_PORT(addr)   (*((unsigned char *)((unsigned int)(addr))))

#if defined (__AT90USB1287__)
	#define MEMORY_OFFSET 0x2100
    #define ADC_VREF_TYPE 0x40
#else
	#define MEMORY_OFFSET 0x1100
    #define ADC_VREF_TYPE 0x00
#endif

#if defined(__USEFARFLASH) || defined(__HAS_ELPM__)
	#define		__FLASH			__farflash
#else // #if defined(__USEFARFLASH) || defined(__HAS_ELPM__)
	#define		__FLASH			__flash
#endif  //	#if defined(__USEFARFLASH) || defined(__HAS_ELPM__)

/* IO defines  */
#define VERSION_1			0
#define VERSION_2			0
#define VERSION_3			0
#define VERSION_4			0

#define BUFER_LENGTH		20
#define DATA_LENGTH_MAX		120
#define HEADER_LENGTH_MAX	6
#define MAX_IO_LENGTH		(HEADER_LENGTH_MAX + DATA_LENGTH_MAX)

#define ADC_BUFER_LENGTH	6

#define STANDARD 0
#define EXPANDED 1

#define PRIVATE_I2C_BUS_ID_2		2
#define PRIVATE_I2C_BUS_ID_3		3

/* Interfases markers */
#define I2C_chanel			0x40
#define SPI_chanel			0x80
#define I2C_PR1_chanel		0x01
#define I2C_PR2_chanel		0x02
#define USB_chanel			0x03
#define OCCUPIED			0xFF
#define FREE				0x00

/* Private I2C messages */
#define IPMI_OK								(0x00)
#define IPMI_LOST_ARBITRATION				(0x81)
#define IPMI_BUS_ERROR						(0x82)
#define IPMI_NAK							(0x83)
#define IPMI_DATA_OUT_OF_RANGE				(0xc9)

/* Messages */
#define INVALID_ADDRESS		0x01
#define INVALID_TARGET		0x02
#define INVALID_CRC			0x03
#define BUFFER_FULL			0x04
#define INVALID_OFFSET      0x05
#define INVALID_CMD			0x06
#define INVALID_LENGTH		0x07
#define TASK_BUSY			0x08
#define INVALID_DIRECTION	0x09
#define INVALID_BUS_ID		0x0A
#define ALL_OK				0x00


/* Commands */
#define ERROR_MESSAGE		0xFF
#define VERSION_MESSAGE		0x01
#define WRITE_REGISTERS		0x02
#define READ_REGISTERS		0x03
#define ADC_READ    		0x04
#define ATPWR_SET    		0x05
#define FAN_SET 			0x06
#define EXT_I2C_READ		0x07
#define EXT_I2C_WRIGHT		0x08
#define EXT_I2C_RESULT		0x09
#define DEBUG_CMD			0xD0
#define PIRO_CMD			0x44

#define FLASH_REC_CMD			0x46
#define FLASH_PLY_CMD			0x47  
#define FLASH_ID_CMD			0x50

#define WRITE_DIR			0x00
#define READ_DIR			0x01
#define MAX_EXT_I2C_LEN		0x40

#if (TARGET_PLAT == PIROPILOT)
    #define  BUSSER_ON   PORTF |= (1<<0)
    #define  BUSSER_OFF  PORTF &= ~(1<<0)
#else
    #define  BUSSER_ON   PORTF &= ~(1<<0)
    #define  BUSSER_OFF  PORTF |= (1<<0)
#endif


#define DIP1_ON  (!(PINC & (1<<0)))
#define DIP1_OFF (PINC & (1<<0))
#define DIP2_ON  (!(PINC & (1<<1)))
#define DIP2_OFF (PINC & (1<<1))
#define DIP3_ON  (!(PINC & (1<<2)))
#define DIP3_OFF (PINC & (1<<2))
#define DIP4_ON  (!(PINC & (1<<3)))
#define DIP4_OFF (PINC & (1<<3))


#define  T_A   0x06
#define  T_OBJ 0x07

typedef struct  {
	unsigned char BusId;
	unsigned char ChipAddress;
	unsigned char Direction;
	unsigned char Length;
	unsigned char Data[MAX_EXT_I2C_LEN];
	
	unsigned char TaskActive;
	unsigned char Error;
} _Ext_I2C_Task_Data;

typedef struct  {
	signed int Ta_p1;
	signed int Tobg_p1;
	signed int Ta_p2;
	signed int Tobg_p2;
        signed int Ta_p3;
	signed int Tobg_p3;
        signed int Ta_p4;
	signed int Tobg_p4;
        signed int  degree_of_bank; // 
        signed int  degree_of_pitch; // 
        signed int  zero_delta; //  
} Piro_IO;



      //! @defgroup USB_key_joy Joystick Management
      //! Macros to manage Joystick on USB KEY
      //! @{
#define Joy_init()            (DDRB &= 0x1F, PORTB |= 0xE0, DDRE &= 0xE7, PORTE |= 0x30)
#define Is_joy_up()           ((PINB & 0x80) ?  FALSE : TRUE)
#define Is_joy_not_up()       ((PINB & 0x80) ?  TRUE  : FALSE)
#define Is_joy_left()         ((PINB & 0x40) ?  FALSE : TRUE)
#define Is_joy_not_left()     ((PINB & 0x40) ?  TRUE : FALSE)
#define Is_joy_select()       ((PINB & 0x20) ?  FALSE : TRUE)
#define Is_joy_not_select()   ((PINB & 0x20) ?  TRUE : FALSE)
#define Is_joy_right()        ((PINE & 0x10) ?  FALSE : TRUE)
#define Is_joy_not_right()    ((PINE & 0x10) ?  TRUE : FALSE)
#define Is_joy_down()         ((PINE & 0x20) ?  FALSE : TRUE)
#define Is_joy_not_down()     ((PINE & 0x20) ?  TRUE : FALSE)

      //! @defgroup USB_key_HWB HWB button management
      //! HWB button is connected to PE2 and can also
      //! be used as generic push button
      //! @{
#define Hwb_button_init()      (DDRE &= 0xFB, PORTE |= 0x04)
#define Is_hwb()               ((PINE & 0x04) ?  FALSE : TRUE)
#define Is_not_hwb()           ((PINE & 0x04) ?  TRUE : FALSE)
      //! @}

#endif	// #ifndef __DEFINE_H_
