
#include <ioavr.h>
// Standard Input/Output functions
#include <stdio.h>

#if (TARGET_PLAT == PIROPILOT)
//      #define SPI_SETTINGS  0x5C
      // DataFlash chip select port pin
      #define DF_CHIP_SELECT 0
      #define DF_CHIP_PORT   PORTB
      #define DF_CHIP_DDR    DDRB
#else
//      #define SPI_SETTINGS  0x5C
      // DataFlash chip select port pin 
      #define DF_CHIP_SELECT 0
      #define DF_CHIP_PORT   PORTE
      #define DF_CHIP_DDR    DDRE
#endif



#define SPIF            7

#define PAUSE  10

#define ENABLEFLASH     DF_CHIP_PORT&=~(1<<DF_CHIP_SELECT)
#define DISABLEFLASH    DF_CHIP_PORT|=(1<<DF_CHIP_SELECT)

#define BUFFER_1 1
#define BUFFER_2 2

// buffer 1 write 
#define BUFFER_1_WRITE 0x84

// buffer 2 write 
#define BUFFER_2_WRITE 0x87

// buffer 1 read
#define BUFFER_1_READ 0xD4

// buffer 2 read
#define BUFFER_2_READ 0xD6

// buffer 1 to main memory page program with built-in erase
#define B1_TO_MM_PAGE_PROG_WITH_ERASE 0x83

// buffer 2 to main memory page program with built-in erase
#define B2_TO_MM_PAGE_PROG_WITH_ERASE 0x86

// buffer 1 to main memory page program without built-in erase
#define B1_TO_MM_PAGE_PROG_WO_ERASE 0x88

// buffer 2 to main memory page program without built-in erase
#define B2_TO_MM_PAGE_PROG_WO_ERASE 0x89

// main memory page program through buffer 1
#define MM_PAGE_PROG_THROUGH_B1 0x82
 
// main memory page program through buffer 2
#define MM_PAGE_PROG_THROUGH_B2 0x85
 
// auto page rewrite through buffer 1
#define AUTO_PAGE_REWRITE_THROUGH_B1 0x58
 
// auto page rewrite through buffer 2
#define AUTO_PAGE_REWRITE_THROUGH_B2 0x59
 
// main memory page compare to buffer 1
#define MM_PAGE_TO_B1_COMP 0x60

// main memory page compare to buffer 2
#define MM_PAGE_TO_B2_COMP 0x61
 
// main memory page to buffer 1 transfer
#define MM_PAGE_TO_B1_XFER 0x53

// main memory page to buffer 2 transfer
#define MM_PAGE_TO_B2_XFER 0x55

// DataFlash status register for reading density, compare status, 
// and ready/busy status
#define STATUS_REGISTER 0x57

// main memory page read
#define MAIN_MEMORY_PAGE_READ 0x52

// erase a 528 byte page
#define PAGE_ERASE 0x81

// erase 512 pages
#define BLOCK_ERASE 0x50



//////////////////////////////////////////////////
extern int uart_usb_putchar(int data_to_send);
extern void uart_usb_flush (void);
extern void DelayMs(unsigned short Ms);

unsigned char read_flash_stat(void);
void writespi (char dataspi);
char readspi(void);

//unsigned char buffer_num = BUFFER_1;





extern void Send_To_USB(char a);
//extern void uart_usb_flush (void);

void erasing (void); 
//   
   
void write_to_flash (unsigned char ad_data);   
//      

void playback (void);      
//      

void next_page_to_next_buffer (unsigned char active_buffer, unsigned int page_counter); 
//       

void active_buffer_to_USB(unsigned char active_buffer);
//      



// global variables  

volatile unsigned char new_data = 0;
//  -  "1"     0- 
//  "0"         


//     0 -   1 - 
volatile unsigned char flash_filed = 0; 


//   ,       
unsigned int last_byte_position_buffer = 0; 

//  ,      
unsigned int last_byte_page = 0;     




void SPI_state(unsigned char st) {
	
	if(st) {
			PORTB &= ~( (1<<0) | (1<<1) | (1<<2) | (1<<3) );
			DDRB  |= ( (1<<0) | (1<<1) | (1<<2) );
			DDRB  &= ~(1<<3);
			
			DF_CHIP_PORT |= (1<<DF_CHIP_SELECT);
			DF_CHIP_DDR  |= (1<<DF_CHIP_SELECT);
                        
              #if (TARGET_PLAT != PIROPILOT)
			// SPI initialization
			// SPI Type: Master
			// SPI Clock Rate: 2*2000,000 kHz
			// SPI Clock Phase: Cycle Start
			// SPI Clock Polarity: High
			// SPI Data Order: MSB First
			SPCR = 0x5C;
			SPSR = 0x01;
              #else
                        // SPI initialization
                        // SPI Type: Master
                        // SPI Clock Rate: 4000,000 kHz
                        // SPI Clock Phase: Cycle Start
                        // SPI Clock Polarity: High
                        // SPI Data Order: MSB First
                        SPCR=0x5C;
                        SPSR=0x00;
              #endif
	} else {
			DF_CHIP_PORT |= (1<<DF_CHIP_SELECT);
			DF_CHIP_DDR  &= ~(1<<DF_CHIP_SELECT);		
		
			SPCR = 0x00; // Disable SPI
			SPSR = 0x00;
		
			PORTB &= ~( (1<<0) | (1<<1) | (1<<2) | (1<<3) );
			DDRB  &= ~( (1<<0) | (1<<1) | (1<<2) | (1<<3) );			
	}
}


//   .
// 1F -  
// 28 -  DB642
unsigned char Is_DB642(void) {
  unsigned char reti = 1;
  
  ENABLEFLASH;
      writespi(0x9F);
      if( readspi() != 0x1F ) reti = 0;
      if( readspi() != 0x28 ) reti = 0;
  DISABLEFLASH;
return reti;
}


//    .       ...
// ,       ...  ...
void erasing(void) {

  ENABLEFLASH;
      writespi(0xC7);
      writespi(0x94);
      writespi(0x80);
      writespi(0x9A);
  DISABLEFLASH;
}

//   DataLen    InputDataBuf 
//        StartAddr
void write_data_to_RAM_page(char *InputDataBuf, unsigned char DataLen, unsigned int StartAddr, char buffer_num) {
  
  while (!(read_flash_stat() & 0x80)); //     
  ENABLEFLASH;
    if(buffer_num == BUFFER_1) writespi(BUFFER_1_WRITE);
                          else writespi(BUFFER_2_WRITE);
    writespi(0x00);
    writespi( (StartAddr >> 8) & 0xFF );
    writespi( StartAddr & 0xFF );
      while( DataLen-- ) writespi( *InputDataBuf++ );
  DISABLEFLASH;  
}

//   DataLen       
//   OutputDataBuf    StartAddr
void read_data_from_RAM_page(char *OutputDataBuf, unsigned char DataLen, unsigned int StartAddr, char buffer_num) {
  
  while (!(read_flash_stat() & 0x80)); //     
  ENABLEFLASH;
    if(buffer_num == BUFFER_1) writespi(BUFFER_1_READ);
                          else writespi(BUFFER_2_READ);
    writespi(0x00);
    writespi( (StartAddr >> 8) & 0xFF );
    writespi( StartAddr & 0xFF );
    writespi(0x00);
    while( DataLen-- ) { 
      *OutputDataBuf++ = readspi();
    }
  DISABLEFLASH;  
}

//     RAM 
//   -  PageAddr
//     .
void RAM_buf_to_flash_page(unsigned int PageAddr, char buffer_num) {
  
    while (!(read_flash_stat() & 0x80)); //     
    ENABLEFLASH;
      if(buffer_num == BUFFER_1) writespi(B1_TO_MM_PAGE_PROG_WITH_ERASE);
                            else writespi(B2_TO_MM_PAGE_PROG_WITH_ERASE);    
      writespi((PageAddr >> 5) & 0xFF);
      writespi((PageAddr << 3) & 0xF8);
      writespi(0x00);
    DISABLEFLASH;  
}

//      PageAddr
//     
void flash_page_to_RAM_buf(unsigned int PageAddr, char buffer_num) {

    while (!(read_flash_stat() & 0x80)); //     
    ENABLEFLASH;
      if(buffer_num == BUFFER_1) writespi(MM_PAGE_TO_B1_XFER);
                            else writespi(MM_PAGE_TO_B2_XFER);    
      writespi((PageAddr >> 5) & 0xFF);
      writespi((PageAddr << 3) & 0xF8);
      writespi(0x00);
    DISABLEFLASH;  
}

/////////////////////////////////////////////////////////////////////




void next_page_to_next_buffer (unsigned char active_buffer, unsigned int page_counter)
{
    unsigned char temp = 0x80;
    
    while (!(read_flash_stat() & 0x80));

   // while(!(PINB & 0x02));                  // wait until flash is not busy
    
    PORTB &= ~DF_CHIP_SELECT;               // enable DataFlash
     
     
    /* :    ,    
       !!! ,   */            
    if (active_buffer == 1)                 // if buffer1 is the active buffer
    {
        SPDR = MM_PAGE_TO_B2_XFER;          // transfer next page to buffer2
    }
    else                                    // else
    {
        SPDR = MM_PAGE_TO_B1_XFER;          // transfer next page to buffer1
    }
    
    
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = (char)(page_counter >> 6);
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = (char)(page_counter << 2);
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed 
    PORTB |= DF_CHIP_SELECT;                // disable DataFlash and start transaction     
}
           
 


void active_buffer_to_USB(unsigned char active_buffer) 
{
    // until active buffer not empty read active buffer to rs232
      
    unsigned int buffer_counter = 0;
    unsigned char temp = 0x80; 
    unsigned char USB_Count = 0;
    
       
    
//    putsf("NextPg."); //  8 
    
       
    PORTB &= ~DF_CHIP_SELECT;               // enable DataFlash     

    if (active_buffer == 1)                 // if buffer1 is the active buffer
    {
        SPDR = BUFFER_1_READ;               // read from buffer1
    }
    else                                    // else
    {
        SPDR = BUFFER_2_READ;               // read from buffer2
    }
    while (!(SPSR & temp));                 // wait for data transfer to be completed                      
    
    
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // start at buffer address 0
    while (!(SPSR & temp));                 // wait for data transfer to be completed
    SPDR = 0x00;                            // write don't care byte
    while (!(SPSR & temp));                 // wait for data transfer to be completed   
         
    while (buffer_counter < 528)                          
    {
  //      SPDR = 0xFF;                        // write dummy value to start register shift
  //      while (!(SPSR & temp));             // wait for data transfer to be completed
        
     

        uart_usb_putchar( readspi() );
        if(USB_Count >= 20) { 
                               uart_usb_flush();
                               USB_Count = 0;
                     } else {
                               USB_Count++;
                            }
   
        buffer_counter++;
     }
     PORTB |= DF_CHIP_SELECT;               // disable DataFlash
}






void writespi (char dataspi) {
  /* Start transmission */
  SPDR = dataspi;
  /* Wait for transmission complete */
  while(!(SPSR & (1<<SPIF)));

}

char readspi(void) {
  SPDR = 0xFF;                        // write dummy value to start register shift
  while (!(SPSR & (1<<SPIF)));             // wait for data transfer to be completed

return SPDR;
}



unsigned char read_flash_stat(void) {
unsigned char stat;

  ENABLEFLASH;

  writespi(0xD7);

  stat = readspi();

  DISABLEFLASH;

return stat;
}

