#include <ioavr.h>
#include <inavr.h>
#include "Private_I2C.h"
#include "Defines.h"

//--------------------------------------------------------------------------------------

//	SCL low stretch timeout in  us
#define SCK_LOW_STRETCH_TIMEOUT			300

static void Private_I2C_Stop(Bus_Descriptor *pBus);
static unsigned char Private_I2C_Send_Message(unsigned char Length, unsigned char Address, unsigned char *Buff, Bus_Descriptor *pBus);
static unsigned char Private_I2C_Receive_Message(unsigned char Length, unsigned char *Buff, Bus_Descriptor *pBus);
static unsigned char check_clock_stretch(unsigned char *sclk_pins, unsigned char sclk_mask);

extern __FLASH Private_Bus_Descriptor Private_Buses[];
//--------------------------------------------------------------------------------------
static void Private_I2C_Init(Bus_Descriptor *pBus, Private_Bus_Descriptor __FLASH *CurrentBus)
{
unsigned char i;
unsigned char interrupt_state;
unsigned char sclk_mask;
unsigned char sdat_mask;
unsigned char *sclk_ddr;

   	pBus->SCLK_Mask = sclk_mask = 1 << ((CurrentBus->SDAT_SCLK_PINS & 0xF0)>>4);
   	pBus->SDAT_Mask = sdat_mask = 1 << (CurrentBus->SDAT_SCLK_PINS & 0x0F);
   	pBus->SCLK_DDR = sclk_ddr = CurrentBus->SCLK_DDR;
   	pBus->SDAT_DDR = CurrentBus->SDAT_DDR;
   	pBus->SCLK_PINS = CurrentBus->SCLK_PINS;
   	pBus->SDAT_PINS = CurrentBus->SDAT_PINS;

	interrupt_state = __save_interrupt();
	__disable_interrupt();

	*(pBus->SCLK_DDR) &= ~sclk_mask; 		//	SET_SCLK_HIGH
	*(pBus->SDAT_DDR) &= ~sdat_mask; 		//	SET_SDAT_HIGH

	*(CurrentBus->SCLK_PORT) &= ~sclk_mask;
	*(CurrentBus->SDAT_PORT) &= ~sdat_mask;
	
	__restore_interrupt(interrupt_state);

    for (i = 0; i < 8; i++) {
		__delay_cycles(DELAY_CLK1);
		interrupt_state = __save_interrupt();
		__disable_interrupt();
		*sclk_ddr |= sclk_mask; 	//	SET_SCLK_LOW
		__restore_interrupt(interrupt_state);
		__delay_cycles(DELAY_CLK0);
		interrupt_state = __save_interrupt();
		__disable_interrupt();
		*sclk_ddr ^= sclk_mask; 	//	SET_SCLK_HIGH
		__restore_interrupt(interrupt_state);
    }
}
//--------------------------------------------------------------------------------------
static void Private_I2C_Stop(Bus_Descriptor *pBus)
{
unsigned char interrupt_state;

 	interrupt_state = __save_interrupt();
 	__disable_interrupt();
	*pBus->SCLK_DDR |= pBus->SCLK_Mask; //	SET_SCLK_LOW
	*pBus->SDAT_DDR |= pBus->SDAT_Mask; //	SET_SDAT_LOW
	__delay_cycles(DELAY_CLK0);
	*pBus->SCLK_DDR &= ~pBus->SCLK_Mask; //	SET_SCLK_HIGH
 	__restore_interrupt(interrupt_state);

	check_clock_stretch(pBus->SCLK_PINS, pBus->SCLK_Mask);
	__delay_cycles(DELAY_STOP);
 	interrupt_state = __save_interrupt();
 	__disable_interrupt();
	*pBus->SDAT_DDR &= ~pBus->SDAT_Mask; //SET_SDAT_HIGH
 	__restore_interrupt(interrupt_state);
	__delay_cycles(DELAY_STOP_START);
	return;
}
//--------------------------------------------------------------------------------------
static unsigned char Private_I2C_Send_Message(unsigned char Length,  unsigned char Address,
	unsigned char *Buff, Bus_Descriptor *pBus) {
unsigned char interrupt_state;
unsigned char Byte;
unsigned char Mult;
unsigned char sclk_mask = pBus->SCLK_Mask;
unsigned char sdat_mask = pBus->SDAT_Mask;
unsigned char *sclk_ddr = pBus->SCLK_DDR;
unsigned char *sdat_ddr = pBus->SDAT_DDR;
unsigned char *sclk_pins = pBus->SCLK_PINS;
unsigned char *sdat_pins = pBus->SDAT_PINS;

 	interrupt_state = __save_interrupt();
 	__disable_interrupt();

	*sclk_ddr &= ~sclk_mask; 							//	SET_SCLK_HIGH
	*sdat_ddr &= ~sdat_mask; 							//	SET_SDAT_HIGH
 	__restore_interrupt(interrupt_state);
	__delay_cycles(DELAY_RESTART);

	if( (!(*sdat_pins & sdat_mask)) || (!(*sclk_pins & sclk_mask))) {
//		debug_print("Private I2C - IPMI_BUS_ERROR\r");
		return IPMI_BUS_ERROR;
	}

 	interrupt_state = __save_interrupt();
 	__disable_interrupt();
	*sdat_ddr |= sdat_mask; 							//	SET_SDAT_LOW
 	__restore_interrupt(interrupt_state);
	__delay_cycles(DELAY_START);

	Byte = Address;
	do {
		Mult = 0x80;
		interrupt_state = __save_interrupt();
		__disable_interrupt();
		while (Mult) {
			*sclk_ddr |= sclk_mask; 					//	SET_SCLK_LOW
			if (Byte & Mult) *sdat_ddr &= ~sdat_mask; 	//	SET_SDAT_HIGH
			else *sdat_ddr |= sdat_mask;				//	SET_SDAT_LOW
			__delay_cycles(DELAY_CLK0);

			*sclk_ddr ^= sclk_mask; 					//	SET_SCLK_HIGH
			__delay_cycles(DELAY_CLK1);

			if ((*sclk_pins & sclk_mask) == 0) {
 				__restore_interrupt(interrupt_state);
//				if((Mult != 0x80) || (check_clock_stretch(sclk_pins, sclk_mask) != IPMI_OK)) {
				if((check_clock_stretch(sclk_pins, sclk_mask) != IPMI_OK)) {
//					debug_print("Private I2C - IPMI_BUS_ERROR\r");
					return IPMI_BUS_ERROR;
				}
				interrupt_state = __save_interrupt();
				__disable_interrupt();
			}

			if ((Byte & Mult) && (!(*sdat_pins & sdat_mask)) ) {
 				__restore_interrupt(interrupt_state);
//				debug_print("Private I2C - IPMI_LOST_ARBITRATION\r");
				return IPMI_LOST_ARBITRATION;
			}
			Mult >>= 1;
		}
		*sclk_ddr |= sclk_mask; 		//	SET_SCLK_LOW
		*sdat_ddr &= ~sdat_mask; 		//	SET_SDAT_HIGH
		__delay_cycles(DELAY_CLK0);

		*sclk_ddr ^= sclk_mask; 		//	SET_SCLK_HIGH
		__delay_cycles(DELAY_CLK1);

		if (!(*sclk_pins & sclk_mask)) {
 			__restore_interrupt(interrupt_state);
//			debug_print("Private I2C - IPMI_BUS_ERROR\r");
			return IPMI_BUS_ERROR;
		}

		// Reading ACK
		if (*sdat_pins & sdat_mask) {
 			__restore_interrupt(interrupt_state);
//			debug_print("Private I2C - IPMI_NAK\r");
			return IPMI_NAK;
		}
		*sclk_ddr |= sclk_mask; 		//	SET_SCLK_LOW
 		__restore_interrupt(interrupt_state);

		Byte = *Buff++;
	} while(Length--);

	return IPMI_OK;
}
//--------------------------------------------------------------------------------------
static unsigned char Private_I2C_Receive_Message(unsigned char Length, unsigned char *Buff, Bus_Descriptor *pBus) {
unsigned char interrupt_state;
unsigned char Value;
unsigned char Mult;
unsigned char sclk_mask = pBus->SCLK_Mask;
unsigned char sdat_mask = pBus->SDAT_Mask;
unsigned char *sclk_ddr = pBus->SCLK_DDR;
unsigned char *sdat_ddr = pBus->SDAT_DDR;
unsigned char *sclk_pins = pBus->SCLK_PINS;
unsigned char *sdat_pins = pBus->SDAT_PINS;

	while (Length--) {
//---------------------------------------------------------------------------
		Value = 0;
		Mult = 0x80;

		interrupt_state = __save_interrupt();
		__disable_interrupt();
		*sclk_ddr |= sclk_mask; 	//	SET_SCLK_LOW
		*sdat_ddr &= ~sdat_mask; 	//	SET_SDAT_HIGH
		while (Mult) {
			__delay_cycles(DELAY_CLK0);

			*sclk_ddr &= ~sclk_mask; //	SET_SCLK_HIGH
			__delay_cycles(DELAY_CLK1);

			if ((*sclk_pins & sclk_mask) == 0) {
 				__restore_interrupt(interrupt_state);
//				if((Mult != 0x80) || (check_clock_stretch(sclk_pins, sclk_mask) != IPMI_OK)) {
				if((check_clock_stretch(sclk_pins, sclk_mask) != IPMI_OK)) {
//					debug_print("Private I2C - IPMI_BUS_ERROR\r");
					return IPMI_BUS_ERROR;
				}
				interrupt_state = __save_interrupt();
				__disable_interrupt();
			}

			if (*sdat_pins & sdat_mask) Value |= Mult;
			Mult >>= 1;

			*sclk_ddr |= sclk_mask; //	SET_SCLK_LOW
		}

		//Send ACK/NACK
		if (Length) *sdat_ddr |= sdat_mask; //	SET_SDAT_LOW
		else *sdat_ddr &= ~sdat_mask; 		//	SET_SDAT_HIGH
		__delay_cycles(DELAY_CLK0);

		*sclk_ddr &= ~sclk_mask; 			//	SET_SCLK_HIGH
		__delay_cycles(DELAY_CLK1);

		if (!(*sclk_pins & sclk_mask)) {
 			__restore_interrupt(interrupt_state);
//			debug_print("Private I2C - IPMI_BUS_ERROR\r");
			return IPMI_BUS_ERROR;
		}

		*Buff++ = Value;
 		__restore_interrupt(interrupt_state);
	}

	return IPMI_OK;
}
//--------------------------------------------------------------------------------------
static unsigned char check_clock_stretch(unsigned char *sclk_pins, unsigned char sclk_mask)
{
unsigned char i, res;
	for (res = i = 0; i < SCK_LOW_STRETCH_TIMEOUT; i++) {
		if ((*sclk_pins) & sclk_mask) {
			res = 1;
			break;
		}
		__delay_cycles(8);
		__watchdog_reset();
	}
	if (res == 0)
		return IPMI_BUS_ERROR;
	return IPMI_OK;
}
//--------------------------------------------------------------------------------------
unsigned char MasterWriteReadProc(  unsigned char BusID, unsigned char I2CAddress,
									unsigned char WriteCount, unsigned char *WriteData,
									unsigned char ReadCount,  unsigned char *ReadData)
{
unsigned char Result;
Private_Bus_Descriptor __FLASH *CurrentBus;
Bus_Descriptor Bus;

	__watchdog_reset();

	if (BusID == 0) {// It`s not a private bus
//		debug_print("Private I2C - IPMI_DATA_OUT_OF_RANGE\r");
		return IPMI_DATA_OUT_OF_RANGE;
	}

	CurrentBus = Private_Buses;
	while (CurrentBus->BusID != BusID) {
		if (CurrentBus->BusID == 0) break;		// Check for end
		CurrentBus++;
	}

	if (CurrentBus->BusID == 0) {
//		debug_print("Private I2C - IPMI_DATA_OUT_OF_RANGE\r");
		return IPMI_DATA_OUT_OF_RANGE;
	}

	Private_I2C_Init(&Bus, CurrentBus);


	// WRITING --------------------------------------------------------
	if(WriteCount || (ReadCount == 0)) {
		Result = Private_I2C_Send_Message(WriteCount, I2CAddress & 0xfe,  WriteData, &Bus);
		Private_I2C_Stop(&Bus);
		if (Result != IPMI_OK) {
			return Result;
		}
	}

	// READING --------------------------------------------------------
	if (ReadCount) {
		if((Result = Private_I2C_Send_Message(0, I2CAddress | 1, WriteData, &Bus)) != IPMI_OK) {
			Private_I2C_Stop(&Bus);
			return Result;
		}

		Result = Private_I2C_Receive_Message(ReadCount, ReadData, &Bus);
		Private_I2C_Stop(&Bus);
	}

	return Result;
}
//--------------------------------------------------------------------------------------
