//! @file scheduler.c,v
//!
//! Copyright (c) 2004 Atmel.
//!
//! Please read file license.txt for copyright notice.
//! @brief This file contains the scheduler routines
//!
//!Configuration:
//!  - SCHEDULER_TYPE in scheduler.h header file
//!  - Task & init for at least task number 1 must be defined
//!
//! @version 1.11 at90usb128-demo-cdc-1_0_3 $Id: scheduler.c,v 1.11 2005/03/10 08:30:50 lguilhau Exp $
//!
//! @todo
//! @bug

//!_____ I N C L U D E S ____________________________________________________
#define _SCHEDULER_C_
#include "..\..\..\at90usb128\demo\cdc\conf\config.h"                         // system definition 
#include "..\..\..\at90usb128\demo\cdc\conf\conf_scheduler.h"            // Configuration for the scheduler
#include "..\..\..\common\modules\scheduler\scheduler.h"                      // scheduler definition 


//!_____ M A C R O S ________________________________________________________
//!_____ D E F I N I T I O N ________________________________________________
#if SCHEDULER_TYPE != SCHEDULER_FREE
//! When SCHEDULER_TYPE != SCHEDULER_FREE, this flag control task calls.
bit   scheduler_tick_flag;
#endif

#ifdef TOKEN_MODE
//! Can be used to avoid that some tasks executes at same time. 
//! The tasks check if the token is free before executing. 
//! If the token is free, the tasks reserve it at the begin of the execution 
//! and release it at the end of the execution to enable next waiting tasks to 
//! run.	
Uchar token;
#endif

//!_____ D E C L A R A T I O N ______________________________________________
//! Scheduler initialization
//!
//! Task_x_init() and Task_x_fct() are defined in config.h
//!
//! @warning Code:XX bytes (function code length)
//!
//! @param  :none
//! @return :none

void Eeprom_Task_Init(void);
void Eeprom_Task_Run(void);

void scheduler_init (void)
{
	usb_task_init();
	cdc_task_init();  
	Eeprom_Task_Init();
}

//! Task execution scheduler
//!
//! @warning Code:XX bytes (function code length)
//!
//! @param  :none
//! @return :none
void scheduler_tasks (void)
{
	// To avoid uncalled segment warning if the empty function is not used
	scheduler_empty_fct();
	
	for(;;) {
		__watchdog_reset();
		usb_task();
		cdc_task();
		Eeprom_Task_Run();
	}
}

//! Init & run the scheduler
//!
//! @warning Code:XX bytes (function code length)
//!
//! @param  :none
//! @return :none
void scheduler (void)
{
   scheduler_init();
   scheduler_tasks();
}


//! Do nothing
//! Avoid uncalled segment warning if the empty function is not used
//!
//! @warning Code:XX bytes (function code length)
//!
//! @param  :none
//! @return :none
void scheduler_empty_fct (void)
{
}

